@@ -42,6 +42,7 @@ void ErrorMonitor::Reset() {
     failure_message_strings_.clear();
     desired_message_strings_.clear();
     ignore_message_strings_.clear();
+    allowed_message_strings_.clear();
     other_messages_.clear();
 }
 
@@ -56,6 +57,12 @@ void ErrorMonitor::SetDesiredFailureMsg(const VkFlags msgFlags, const char *cons
     test_platform_thread_unlock_mutex(&mutex_);
 }
 
+void ErrorMonitor::SetAllowedFailureMsg(const char *const msg) {
+    test_platform_thread_lock_mutex(&mutex_);
+    allowed_message_strings_.emplace_back(msg);
+    test_platform_thread_unlock_mutex(&mutex_);
+}
+
 void ErrorMonitor::SetUnexpectedError(const char *const msg) {
     test_platform_thread_lock_mutex(&mutex_);
 
@@ -96,6 +103,16 @@ VkBool32 ErrorMonitor::CheckForDesiredMsg(const char *const msgString) {
             }
         }
 
+        if (!found_expected && allowed_message_strings_.size()) {
+            for (auto allowed_msg_it = allowed_message_strings_.begin(); allowed_msg_it != allowed_message_strings_.end();
+                 ++allowed_msg_it) {
+                if (errorString.find(*allowed_msg_it) != string::npos) {
+                    found_expected = true;
+                    break;
+                }
+            }
+        }
+
         if (!found_expected) {
             printf("Unexpected: %s\n", msgString);
             other_messages_.push_back(errorString);
@@ -1611,7 +1628,7 @@ VkShaderObj::VkShaderObj(VkDeviceObj *device, const char *shader_code, VkShaderS
     m_stage_info.pSpecializationInfo = specInfo;
 
     std::vector<unsigned int> spv;
-    framework->GLSLtoSPV(stage, shader_code, spv, debug, spirv_minor_version);
+    framework->GLSLtoSPV(&device->props.limits, stage, shader_code, spv, debug, spirv_minor_version);
 
     VkShaderModuleCreateInfo moduleCreateInfo = {};
     moduleCreateInfo.sType = VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO;
